/* BOREXINO Reconstruction program
 *
 * Author: Davide D'Angelo <davide.dangelo@lngs.infn.it>
 * based on work by Razeto&Pallas 
 *
 * $Id: bx_laben_event.hh,v 1.2 2004/04/27 16:58:57 ddangelo Exp $
 *
 * The laben event object
 * 
 */
#ifndef _BX_LABEN_EVENT_HH
#define _BX_LABEN_EVENT_HH

#include <vector>

#include "bx_rec_general.hh"
#include "constants.hh"

class bx_laben_raw_hit {
  public:
    bx_laben_raw_hit (const char *disk_hit);
    unsigned short get_channel      ()  const { return u2_channel; }
    unsigned char  get_time_1       ()  const { return u1_time_1; }
    unsigned char  get_time_2       ()  const { return u1_time_2; }
    unsigned short get_gray_counter ()  const { return u2_gray_counter; }
    unsigned char  get_base         ()  const { return u1_base; }
    unsigned char  get_peak         ()  const { return u1_peak; }
    unsigned short get_flags        ()  const { return u2_flags; }
    unsigned short get_errors       ()  const { return u2_errors; }
  private:
    unsigned short u2_channel;
    unsigned char  u1_time_1, u1_time_2;
    unsigned short u2_gray_counter;
    unsigned char  u1_base, u1_peak;
    unsigned short u2_flags;
    unsigned short u2_errors;
};

class bx_laben_raw_event {
  public:
    bx_laben_raw_event (const char *disk_event);
    virtual ~bx_laben_raw_event () {}
    const bx_laben_raw_hit& get_raw_hit (int i) const { return raw_hits[i]; }
    int get_raw_nhits () const { return raw_hits.size (); }
  private:
    unsigned long u4_entries;
    unsigned long u4_errors;
    typedef std::vector<bx_laben_raw_hit> bx_laben_raw_hit_vector;
    bx_laben_raw_hit_vector raw_hits;
};


// class bx_laben_decoded_hit {
//   public:
//     bx_laben_decoded_hit (int index);
//     const bx_laben_raw_hit& get_laben_raw_hit () const;
//     float get_decoded_time ()   const { return f4_time; }
//     float get_charge ()         const { return f4_charge; }
//     float get_uncorrected_charge () const;
// //      { return (get_laben_raw_hit ().get_peak () - get_laben_raw_hit ().base ()) * f4_bin_to_charge; }
//     float get_d80 ()            const { return f4_d80; }
//     float get_slope_error ()    const { return f4_slope_error; }
//     unsigned short get_guess () const { return u2_guess; }
//   private:
//     static float f4_bin_to_charge;
//     float f4_charge, f4_time;
//     float f4_d80, f4_slope_error;
//   unsigned short u2_guess;
// }; 

class bx_laben_event : 
  public bx_laben_raw_event {
public:
  bx_laben_event (const char *disk_event): bx_laben_raw_event(disk_event) {}
  virtual ~bx_laben_event () {}
};

#endif
/*
 * $Log: bx_laben_event.hh,v $
 * Revision 1.2  2004/04/27 16:58:57  ddangelo
 * minor updates
 *
 * Revision 1.1  2004/04/27 09:41:09  ddangelo
 * Event structure redisegned.
 * bx_reco_event now handles event header and has object for different detector portions.
 * Inheritance from different reconstruction status is kept within each detector portion obj.
 * Hit lists are linked by const ptr (maybe a temporary solution).
 *
 *
 */
