/* BOREXINO Reconstruction program
 *
 * Author: Davide D'Angelo <davide.dangelo@lngs.infn.it>
 * based on work by Razeto&Pallas 
 *
 * $Id: bx_reco_event.hh,v 1.9 2004/04/28 11:03:38 ddangelo Exp $
 *
 * This is the generic event object
 *
 */

#ifndef _BX_RECO_EVENT_HH
#define _BX_RECO_EVENT_HH

#include "bx_rec_general.hh"

#include "bx_trigger_event.hh"
#include "bx_laben_event.hh"
#include "bx_muon_event.hh"

class bx_reco_event {
  public:
    bx_reco_event (const char *disk_event);
    virtual ~bx_reco_event () {}

    unsigned long get_event_size_bytes   () const { return u4_event_size_bytes; }
    unsigned long get_run_number   () const { return u4_run_number; }
    unsigned long get_event_number () const { return u4_event_number; }
    bool is_crate_enabled_daq (unsigned char c) const { if (c==0) return true; 
                                                  if (c <=16) return u4_enabled_crates & (1 << (c-1)); 
						  else return false; }
    bool is_crate_enabled_runtime (unsigned char c) const { if (c==0) return true; 
                                                       if (c <=16) return u4_enabled_crates & (1 << (15+c)); 
						       else return false; }
    bool is_laben_enabled () const { bool ret_val = false; 
                                    for (int i=1; i<constants::laben.ncrates; i++) ret_val |= is_crate_enabled_runtime(i); 
				    return ret_val;}
    bool is_muon_enabled () const { return is_crate_enabled_runtime(constants::muon.crate_number); }
    bool is_fadc_enabled () const { return is_crate_enabled_runtime(constants::fadc.crate_number); }

    unsigned long get_builder_time_seconds () const { return u4_builder_time_seconds; }
    unsigned long get_builder_time_nanoseconds () const { return u4_builder_time_nanoseconds; }

		
    // getters for different sub-objects
    // Two functions are supplied for each sub-obj:
    // 1) The const getter should be used for reading purposes.
    // 2) The nonconst getter should be used in the module dynamic cast for writing purposes.
    const bx_trigger_event& get_trigger () {return trigger; }
    const bx_laben_event& get_laben () {return laben; }
    const bx_muon_event& get_muon () {return muon; }
    bx_trigger_event& get_trigger_nonconst () {return trigger; }
    bx_laben_event& get_laben_nonconst () {return laben; }
    bx_muon_event& get_muon_nonconst () {return muon; }
		
  private:
    unsigned long u4_event_size_bytes;
    unsigned long u4_run_number;
    unsigned long u4_event_number;
    unsigned long u4_enabled_crates;
    unsigned long u4_builder_time_seconds;
    unsigned long u4_builder_time_nanoseconds;

    bx_trigger_event trigger;
    bx_laben_event laben;
    bx_muon_event muon;
};

#endif

/*
 * $Log: bx_reco_event.hh,v $
 * Revision 1.9  2004/04/28 11:03:38  ddangelo
 * implemented 2 (const/nonconst) getters for sub-objects, for reading/writing within modules
 *
 * Revision 1.8  2004/04/27 13:35:48  ddangelo
 * debugging
 *
 * Revision 1.7  2004/04/27 13:24:06  ddangelo
 * a few more fancy getters
 *
 * Revision 1.6  2004/04/27 12:13:08  ddangelo
 * added getters for enabled_crates and builder times
 *
 * Revision 1.5  2004/04/27 09:41:09  ddangelo
 * Event structure redisegned.
 * bx_reco_event now handles event header and has object for different detector portions.
 * Inheritance from different reconstruction status is kept within each detector portion obj.
 * Hit lists are linked by const ptr (maybe a temporary solution).
 *
 * Revision 1.4  2004/03/23 15:31:57  razeto
 * Added bx_decoded_event inheritance to bx_reco_event
 *
 * Revision 1.3  2004/03/21 18:27:19  razeto
 * Fixed a typo in the log tag
 *
 *
 */
