/* BOREXINO Reconstruction program
 *
 * Author: Marco Pallavicini <pallas@ge.infn.it>
 *
 * $Id: bx_trigger_event.hh,v 1.2 2004/04/27 10:45:23 ddangelo Exp $
 *
 * The trigger event object
 * 
 */
#ifndef _BX_TRIGGER_EVENT_HH
#define _BX_TRIGGER_EVENT_HH

#include "bx_rec_general.hh"
#include "constants.hh"

class bx_trigger_raw_event {
  public:
    bx_trigger_raw_event (const char *disk_trigger_event);
    unsigned short get_length   () const { return u2_length; }
    unsigned short get_error    () const { return u2_error; }    
    unsigned short get_blocks   () const { return u2_blocks; }
    unsigned short get_version  () const { return u2_version; } 
    unsigned long  get_conf1    () const { return u4_conf1; }
    unsigned long  get_conf2    () const { return u4_conf2; }
    unsigned long  get_time     () const { return u4_time; }
    unsigned long  get_evid     () const { return u4_evid; }
    unsigned long  get_trgw     () const { return u4_trgw; }
    unsigned long  get_gps1     () const { return u4_gps1; }
    unsigned long  get_gps2     () const { return u4_gps2; }
    unsigned long  get_gps3     () const { return u4_gps3; }
  private:
    unsigned short u2_length;
    unsigned short u2_error;
    unsigned short u2_blocks;
    unsigned short u2_version;
    unsigned long  u4_conf1;
    unsigned long  u4_conf2;
    unsigned long  u4_time;
    unsigned long  u4_evid;
    unsigned long  u4_trgw;
    unsigned long  u4_gps1;
    unsigned long  u4_gps2;
    unsigned long  u4_gps3;
};

class bx_trigger_decoded_event {
  public:
    bx_trigger_decoded_event () {}
   
    // getters
    int get_day      () const { return i4_day; }
    int get_mon      () const { return i4_mon; }
    int get_year     () const { return i4_year; }
    int get_hour     () const { return i4_hour; }
    int get_min      () const { return i4_min; }
    int get_sec      () const { return i4_sec; }
    int get_millisec () const { return i4_millisec; }
    int get_microsec () const { return i4_microsec; }

    // return event time in seconds, and microseconds since Jan 01, 2000 0:0:0;
    void get_time    (unsigned long &secs, unsigned long &micro) const {secs=i4_time1;  micro=i4_time2; }

  private:
    int i4_day, i4_mon, i4_year;
    int i4_hour, i4_min, i4_sec, i4_millisec, i4_microsec;
    unsigned long i4_time1, i4_time2;  // absolute secs and microsecs since 01/01/200 
    
  friend class bx_trigger_decoder;
};

class bx_trigger_event: 
  public bx_trigger_raw_event, 
  public bx_trigger_decoded_event {
  public:
    bx_trigger_event (const char *disk_event): bx_trigger_raw_event(disk_event) {}
    virtual ~bx_trigger_event () {}
};

#endif
/*
 * $Log: bx_trigger_event.hh,v $
 * Revision 1.2  2004/04/27 10:45:23  ddangelo
 * fixed module name in friend declaration
 *
 * Revision 1.1  2004/04/27 09:41:09  ddangelo
 * Event structure redisegned.
 * bx_reco_event now handles event header and has object for different detector portions.
 * Inheritance from different reconstruction status is kept within each detector portion obj.
 * Hit lists are linked by const ptr (maybe a temporary solution).
 *
 *
 */
