/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: laben_time_hit.hh,v 1.3 2004/04/27 09:41:09 ddangelo Exp $
 *
 * A class to decode the laben hit time
 *
 */
#ifndef _LABEN_TIME_HIT
#define _LABEN_TIME_HIT

#include "bx_laben_event.hh"

class laben_time_hit {
  public:
    enum ramp_slope { // Do not change the values!!!
      rising = 0,
      falling = 1,
    };

    laben_time_hit (const bx_laben_raw_hit& hit);
    
    ramp_slope guess_slope ();
    ramp_slope hw_slope () const { return i_hw_slope; } 
    float get_time ();
    float get_d80 ();
    float get_error () const { return f_error; }

  private:
    float f_d80;
    int i_low, i_high;
    float crate_time_correction;
    bool rising_on_even;
    int i_gr_shift;
      // some constants
    float f_ramp_slope, f_t20, f_t30;

    void m_calc_ramp_times (ramp_slope slope, float& t1, float& t2);

    unsigned char u1_ramp_1, u1_ramp_2;
    unsigned short u2_gray_count;
    ramp_slope i_hw_slope;

    float f_error;

    static const float ramp_length_ns;
    static const float window_upper_bound;
    static const float window_lower_bound;
    static const unsigned char spire_threshold;
    static const float d80;
};


#endif
/*
 * $Log: laben_time_hit.hh,v $
 * Revision 1.3  2004/04/27 09:41:09  ddangelo
 * Event structure redisegned.
 * bx_reco_event now handles event header and has object for different detector portions.
 * Inheritance from different reconstruction status is kept within each detector portion obj.
 * Hit lists are linked by const ptr (maybe a temporary solution).
 *
 * Revision 1.2  2004/04/24 17:37:40  razeto
 * Added a method for calculating d80
 *
 * Revision 1.1  2004/04/15 13:00:14  razeto
 * Added
 *
 */
