/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: bx_options.hh,v 1.4 2004/04/06 12:39:28 razeto Exp $
 *
 * Options interface
 *
 */
#ifndef _BX_OPTIONS_H
#define _BX_OPTIONS_H

#include "bx_reco_framework.hh"
#include "bx_rec_general.hh"

#include <string>
#include <vector>

class bx_options {
  public:
    static bx_options* get ();
    void set_command_line (int argc, char * const argv[]);
    void set_module_dot_parameter_value (const std::string& module_dot_parameter, const std::string& value);

    const std::string&  get_config_filename () const { return s_config_filename; }
    const std::string&  get_module_list_filename () const { return s_module_list_filename; }
    const std::string&  get_log_filename () const { return s_log_filename; }
  private:
    bx_options ();
    static bx_options *me;
    std::string s_config_filename, s_module_list_filename;
    std::string s_log_filename;
    void m_read_config_file (bool);

      // The following modules, structure and vector are used because I want
      // the parameter read from file has lower priority than the ones on the
      // command line. For this reason I commit parameter set instructions in a vector
      // then I read the config file, which add parameter to the tail of the vector.
      // At commit the vector is read in backward direction.
    void m_schedule_set_module_dot_parameter_value (const std::string& module_dot_parameter, const std::string& value);
    void m_update_module_dot_parameter_value ();
      // Defininig a structure inside a class is safe, and is done since I want
      // it could be used ONLY by the class members.
    struct scheduled_parameter_set { 
      scheduled_parameter_set (const std::string& module_dot_parameter, const std::string& value): s_module_dot_parameter(module_dot_parameter), s_value(value) {} 
      std::string s_module_dot_parameter, s_value; 
    };
    std::vector<scheduled_parameter_set> scheduled_sets;
};
 
#endif
/*
 * $Log: bx_options.hh,v $
 * Revision 1.4  2004/04/06 12:39:28  razeto
 * Added logfile handling
 *
 * Revision 1.3  2004/04/05 13:12:27  razeto
 * Updated a comment, changed the behaviour if no user option file is present
 *
 * Revision 1.2  2004/03/29 13:17:30  razeto
 * Updated bx_options to the vdt usage
 *
 * Revision 1.1  2004/03/26 16:36:11  razeto
 * Introduced bx_options; a lot of code modified to read options and parameters.
 * Bx_event_reader interface changed: now there is a standard constructor and
 * the file opening is done at begin using the parameters for the file name.
 *
 * 
 */
