/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: db_profile.hh,v 1.3 2004/04/26 13:48:29 razeto Exp $
 *
 * The database interface for profile informations
 *
 */
#ifndef _BD_PROFILE_H
#define _BD_PROFILE_H
#include "bx_rec_general.hh"
#include "cmap.hh"
#include "db_acl.hh"

#include <string>
#include <map>
#include <vector>

class db_profile: public db_acl {
  public:
    enum channel_description_type {
      // general description
      empty = 0,
      ordinary = 1,
      trigger = 2,
      laser = 3,
      // laben description
      pulser = 3,
      // fadc descriptions
      total_sum = 10,
      attenuated_sum = 11,
      trigger_ramp = 12,
      // nuon descriptions
      integrity = 20,
      clock = 21,
    };
    struct pmt_coordinates { float x, y, z; };
    
    channel_description_type logical_channel_description (int lg) const { return logical_channel_description_v[lg]; }

    float laben_crate_delay		(int crate) const { return laben_crate_delay_v[crate]; }
    
    long int neutrino_trigger_tag 	() const { return i4_neutrino_trigger_tag; }
    long int muon_mtb_trigger_tag 	() const { return i4_muon_mtb_trigger_tag; }
    long int muon_tct_trigger_tag	() const { return i4_muon_tct_trigger_tag; }
    long int laser266_trigger_tag 	() const { return i4_laser266_trigger_tag; }
    long int laser355_trigger_tag 	() const { return i4_laser355_trigger_tag; }
    long int laser394_trigger_tag 	() const { return i4_laser394_trigger_tag; }
    long int pulser_trigger_tag 	() const { return i4_pulser_trigger_tag; }
    long int random_trigger_tag 	() const { return i4_random_trigger_tag; }
    float trigger_start_gate		() const { return f_trigger_start_gate; }
    float trigger_end_gate		() const { return f_trigger_end_gate; }
    float laser394_trigger_start_gate	() const { return f_laser394_trigger_start_gate; }
    float laser394_trigger_end_gate	() const { return f_laser394_trigger_end_gate; }
    
    int number_of_internal_pmt 		() const { return i_number_of_internal_pmt; }
    int number_of_external_pmt 		() const { return i_number_of_external_pmt; }

    bool pmt_has_cone 			(int lg) const { return pmt_has_cone_v[lg]; }
    int pmt_hole_id 			(int lg) const { return pmt_hole_id_v[lg]; }
    int pmt_fadc_channel 		(int lg) const { return pmt_fadc_channel_v[lg]; }
    const pmt_coordinates& get_pmt_coordinates (int lg) const { return pmt_coordinates_v[lg]; }

    float stainless_steel_sphere_radius	() const { return f_stainless_steel_sphere_radius; }
    float inner_vessel_radius 		() const { return f_inner_vessel_radius; }
    float light_collector_radius 	() const { return f_light_collector_radius; }
    float light_guide_exit_aperture 	() const { return f_light_guide_exit_aperture; }
    float light_guide_entry_aperture	() const { return f_light_guide_entry_aperture; }
    float light_guide_lenght		() const { return f_light_guide_lenght; }
    float pmt_chathode_radius		() const { return f_pmt_chathode_radius; }
    float pmt_bulb_radius		() const { return f_pmt_bulb_radius; }

    float pmt_time_jitter		() const { return f_pmt_time_jitter; }
    float pmt_reflectivity		() const { return f_pmt_reflectivity; }
    float light_guide_reflectivity	() const { return f_light_guide_reflectivity; }
    float sss_specular_reflectivity	() const { return f_sss_specular_reflectivity; }
    float sss_diffusion_reflectivity	() const { return f_sss_diffusion_reflectivity; }
    float sss_uni_reflectivity		() const { return f_sss_uni_reflectivity; }

  private:
    std::cmap<std::string, channel_description_type> channel_description_type_map;
    std::cmap<int, channel_description_type> logical_channel_description_v;
    
    std::cmap<int, float> laben_crate_delay_v;
    
    long int i4_neutrino_trigger_tag;
    long int i4_muon_mtb_trigger_tag;
    long int i4_muon_tct_trigger_tag;
    long int i4_laser266_trigger_tag;
    long int i4_laser355_trigger_tag;
    long int i4_laser394_trigger_tag;
    long int i4_pulser_trigger_tag;
    long int i4_random_trigger_tag;
    float f_trigger_start_gate;
    float f_trigger_end_gate;
    float f_laser394_trigger_start_gate;
    float f_laser394_trigger_end_gate;

    int i_number_of_internal_pmt;
    int i_number_of_external_pmt;
    
    float f_stainless_steel_sphere_radius;
    float f_inner_vessel_radius;
    float f_light_collector_radius;
    float f_light_guide_exit_aperture;
    float f_light_guide_entry_aperture;
    float f_light_guide_lenght;
    float f_pmt_chathode_radius;
    float f_pmt_bulb_radius;

    float f_pmt_time_jitter;
    float f_pmt_reflectivity;
    float f_light_guide_reflectivity;
    float f_sss_specular_reflectivity;
    float f_sss_diffusion_reflectivity;
    float f_sss_uni_reflectivity;
    
    
    std::cmap<int, bool> pmt_has_cone_v;
    std::cmap<int, int> pmt_hole_id_v, pmt_fadc_channel_v;
    std::cmap<int, pmt_coordinates> pmt_coordinates_v;

    void m_load_laben_channel_mapping (int profile_id);
    void m_load_fadc_channel_mapping (int profile_id);
    void m_load_muon_channel_mapping (int profile_id);
    void m_load_detector_geometry (int profile_id);
    void m_load_profile_table (int profile_id);
    void m_load_physics_constants (int profile_id);

    db_profile (int profile_id);
    db_profile (const db_profile& r) {} // I do not want copy ctor
    friend class bx_dbi;
};
#endif
/*  
 *  $Log: db_profile.hh,v $
 *  Revision 1.3  2004/04/26 13:48:29  razeto
 *  Added db_acl to check set calls for calling module to have the right privileges.
 *  Fixed the names of set/get methods.
 *  Modifications decided at the software Paris meeting.
 *
 *  Revision 1.2  2004/04/12 16:08:19  razeto
 *  Added crate delay fixed map
 *
 *  Revision 1.1  2004/04/09 08:05:00  razeto
 *  Added
 *
 */
