/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: db_run.cc,v 1.2 2004/04/26 13:48:29 razeto Exp $
 *
 * The database interface for run informations
 *
 */
#include "db_run.hh"
#include "bx_dbi.hh"
#include "messenger.hh"
#include "date.hh"

#include <sstream>

db_run::db_run (int run_number) {

  set_acl ("set_laben_precalib_low_bin", "bx_precalib_laben_adc");
  set_acl ("set_laben_precalib_high_bin", "bx_precalib_laben_adc");
  set_acl ("set_laben_precalib_gray_shift", "bx_precalib_laben_gray");
  set_acl ("set_laben_precalib_rising_on_even", "bx_precalib_laben_phases");
  set_acl ("set_laben_precalib_delta80", "bx_precalib_laben_d80");
  set_acl ("set_muon_precalib_pulse_time", "bx_precalib_muon_findpulse");
  set_acl ("set_muon_precalib_pedestal", "bx_precalib_muon_pedestals");

  run_type_map["laser"] = db_run::laser;
  run_type_map["pulser"] = db_run::pulser;
  run_type_map["normal"] = db_run::normal;

  bx_dbi *dbi = bx_dbi::get ();
  bx_message &msg = bx_dbi::get ()->get_message (bx_message::critic);
  msg << "db_run: ";

  std::ostringstream t;
  t << "\"RunNumber\"=" << run_number;
  const bx_dbi::table &table = dbi->query (bx_dbi::daq_config, "\"Run\"", t.str (), "*", "", -1);

  i_run_number = table["RunNumber"][0].get_int ();
  if (i_run_number != run_number) 
    msg << "database corruption asked run " << run_number << " received run " << i_run_number << dispatch;

  i_profile_id	= table["ProfileID"][0].get_int ();

  if (!run_type_map.check (table["Type"][0].get_string())) msg << "unknown run type " << table["Type"][0] << dispatch;
  else i_run_type = run_type_map[table["Type"][0].get_string()];

  i_max_time	= table["MaxTime"][0].get_int ();
  i_max_events	= table["MaxEvent"][0].get_int ();
  i_stop_reason	= table["StopReason"][0].get_int ();
  i_number_of_files = table["NumberOfFiles"][0].get_int ();
  u4_start_time	= std::date(table["StartTime"][0].get_string ()).time();
  u4_stop_time	= std::date(table["StopTime"][0].get_string ()).time ();
  i_duration	= table["Duration"][0].get_int ();
  i_events	= table["Events"][0].get_int ();

  bx_dbi::get ()->get_message (bx_message::info) << "db_run: db_run (" << run_number << ") initialized" << dispatch;
}
/*  
 *  $Log: db_run.cc,v $
 *  Revision 1.2  2004/04/26 13:48:29  razeto
 *  Added db_acl to check set calls for calling module to have the right privileges.
 *  Fixed the names of set/get methods.
 *  Modifications decided at the software Paris meeting.
 *
 *  Revision 1.1  2004/04/09 08:05:00  razeto
 *  Added
 *
 */
