/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: db_run.hh,v 1.5 2004/04/26 13:48:29 razeto Exp $
 *
 * The database interface for run informations
 *
 */
#ifndef _BD_RUN_H
#define _BD_RUN_H
#include "bx_rec_general.hh"
#include "cmap.hh"
#include "db_acl.hh"

#include <string>
#include <map>
#include <vector>
#include <time.h>

class db_run: public db_acl {
  public:
    enum run_type {
      normal,
      laser,
      pulser,
    };

    int get_number	 	() const { return i_run_number; }
    int get_profile_id	 	() const { return i_profile_id; }
    int get_calib_profile 	() const { return i_calib_profile; }
    run_type get_type	 	() const { return i_run_type; }
    int get_max_time	 	() const { return i_max_time; }
    int get_max_events 		() const { return i_max_events; }
    int get_stop_reason		() const { return i_stop_reason; }
    int get_number_of_files 	() const { return i_number_of_files; }
    time_t get_start_time	() const { return u4_start_time; }
    time_t get_stop_time	() const { return u4_stop_time; }
    int get_duration 		() const { return i_duration; }
    int get_events 		() const { return i_events; }

    std::string data_file_url () const { return "file://null"; }

      // S/Getters for the precalib data: a check method is also supplied to
      // check if the data is available (ie the precalib for the required data
      // has already run or the data are already stored on db).
    int get_laben_precalib_low_bin  	(int lg) const { return laben_precalib_low_bin_v[lg]; }
    int get_laben_precalib_high_bin 	(int lg) const { return laben_precalib_high_bin_v[lg]; }
    bool get_laben_precalib_rising_on_even (int lg) const { return laben_precalib_rising_on_even_v[lg]; }
    int get_laben_precalib_gray_shift	(int lg) const { return laben_precalib_gray_shift_v[lg]; }
    float get_laben_precalib_delta80 	(int lg) const { return laben_precalib_delta80_v[lg]; }
    bool check_laben_precalib_low_bin	(int lg) const { return laben_precalib_low_bin_v.check (lg); }
    bool check_laben_precalib_delta80	(int lg) const { return laben_precalib_delta80_v.check (lg); }
    bool check_laben_precalib_high_bin	(int lg) const { return laben_precalib_high_bin_v.check (lg); }
    bool check_laben_precalib_rising_on_even (int lg) const { return laben_precalib_rising_on_even_v.check (lg); }
    bool check_laben_precalib_gray_shift(int lg) const { return laben_precalib_gray_shift_v.check (lg); }
//    const b_vector& laben_alive     () const { return laben_alive; }
    void set_laben_precalib_low_bin	(int lg, int count, const bx_base_module* module) { if (check_acl ("set_laben_precalib_low_bin", module)) laben_precalib_low_bin_v[lg] = count; }
    void set_laben_precalib_high_bin	(int lg, int count, const bx_base_module* module) { if (check_acl ("set_laben_precalib_high_bin", module)) laben_precalib_high_bin_v[lg] = count; }
    void set_laben_precalib_gray_shift	(int lg, int shift, const bx_base_module* module) { if (check_acl ("set_laben_precalib_gray_shift", module)) laben_precalib_gray_shift_v[lg] = shift; }
    void set_laben_precalib_delta80	(int lg, float dt, const bx_base_module* module)  { if (check_acl ("set_laben_precalib_delta80", module)) laben_precalib_delta80_v[lg] = dt; }

      // Muon stuff
    unsigned long get_muon_precalib_pulse_time () const { return u4_muon_precalib_pulse_time; }
    double get_muon_precalib_pedestal	(int lg) const { return muon_precalib_pedestal_v[lg]; } 
    bool check_muon_precalib_pedestal	(int lg) const { return muon_precalib_pedestal_v.check (lg); } 
    void set_muon_precalib_pulse_time	(unsigned long time, const bx_base_module* module) { if (check_acl ("set_muon_precalib_pulse_time", module)) u4_muon_precalib_pulse_time = time; }
    void set_muon_precalib_pedestal	(int lg, double pedestal, const bx_base_module* module) { if (check_acl ("set_muon_precalib_pedestal", module)) muon_precalib_pedestal_v[lg] = pedestal; }

  private:
    int i_run_number;
    int i_profile_id;
    int i_calib_profile;
    run_type i_run_type;
    int i_max_time;
    int i_max_events;
    int i_stop_reason;
    int i_number_of_files;
    time_t u4_start_time;
    time_t u4_stop_time;
    int i_duration;
    int i_events;

      // precalib data
    std::cmap<int, float> laben_precalib_delta80_v;
    std::cmap<int, int> laben_precalib_low_bin_v, laben_precalib_high_bin_v, laben_precalib_gray_shift_v;
    std::cmap<int, bool> laben_precalib_rising_on_even_v;
    unsigned long u4_muon_precalib_pulse_time;
    std::cmap<int, double> muon_precalib_pedestal_v;


    std::cmap<std::string, run_type> run_type_map;

    db_run (int run_number);
    db_run (const db_run& r) {} // I do not want copy ctor 
    friend class bx_dbi;
};
#endif
/*  
 *  $Log: db_run.hh,v $
 *  Revision 1.5  2004/04/26 13:48:29  razeto
 *  Added db_acl to check set calls for calling module to have the right privileges.
 *  Fixed the names of set/get methods.
 *  Modifications decided at the software Paris meeting.
 *
 *  Revision 1.4  2004/04/24 17:41:11  razeto
 *  Added a void method. To be developed
 *
 *  Revision 1.3  2004/04/20 11:37:31  ddangelo
 *  added a few muon precalibration varaibles and relative sgetters.
 *
 *  Revision 1.2  2004/04/12 16:10:58  razeto
 *  Updated
 *
 *  Revision 1.1  2004/04/09 08:05:00  razeto
 *  Added
 *
 */
