/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: messenger.cc,v 1.7 2004/04/18 10:30:14 razeto Exp $
 *
 * Implementation of messenger and bx_message
 *
 */
#include "messenger.hh"
#include "bx_options.hh"

messenger* messenger::me = 0;

messenger* messenger::get () {
  if (!me) 
    me = new messenger;

  return me;
}

messenger::messenger() {
  // Open log file
  const std::string &str = bx_options::get ()->get_log_filename ();
  if (str.size ()) {
    log.open (str.c_str ());
    if (!log) throw std::runtime_error("Can not open file " + str + " for logging");
  }
}


void messenger::submit (bx_message &msg) {
  if (msg.str ().substr (msg.str ().size() - 1, 1) != std::string("\n")) msg << std::endl;

  if (msg.check_print_level ()) 
    std::cout << ">>> " << msg.level () << ": " << msg.str ();
  if (log && msg.check_log_level ()) 
    log << ">>> " << msg.level () << ": " << msg.str ();
  
  if (msg.check_exception ()) {
    log.close ();
    throw std::runtime_error (msg.str ());
  }
} 

bx_message::message_level bx_message::default_print_level = bx_message::warn;
bx_message::message_level bx_message::default_log_level = bx_message::info;

bx_message::bx_message (message_level level, const std::string& initial_message) { 
  message_codes[bx_message::none] = "none";
  message_codes[bx_message::debug] = "debug";
  message_codes[bx_message::info] = "info";
  message_codes[bx_message::warn] = "warn";
  message_codes[bx_message::error] = "error";
  message_codes[bx_message::critic] = "critic";

  clear ();
  i4_message_level = level;
  if (initial_message.size ()) *this << initial_message;
}


bx_message::message_level bx_message::m_decode (const vdt& v) const {

  bx_message::message_level ret_val = bx_message::none;
  
  bx_message error_msg(bx_message::warn, "bx_message: ");

  if (v.get_type () == vdt::string_vdt) {
    message_code_map::const_iterator it = message_codes.rfind (v.get_string ());
    if (it == message_codes.end ()) {
      error_msg << "can not find message level for \"" << v <<"\", assuming \"none\"";
      error_msg.dispatch ();
    } else ret_val = it->first;
  } else if (v.get_type () == vdt::int_vdt) {
    if (v.get_int () < bx_message::none || v.get_int () > bx_message::critic) {
      error_msg << "can not find message level for \"" << v <<"\", assuming \"none\""; 
      error_msg.dispatch ();
    } else ret_val = bx_message::message_level(v.get_int ());
  } else {
    error_msg << "can not find message level for \"" << v <<"\", assuming \"none\""; 
    error_msg.dispatch ();
  }
    
  return ret_val;
}

/*
 * $Log: messenger.cc,v $
 * Revision 1.7  2004/04/18 10:30:14  razeto
 * Fixed to compile wit g++ 2.95 present on the cluster
 *
 * Revision 1.6  2004/04/12 16:07:40  razeto
 * Added configurable default print level for messages
 *
 * Revision 1.5  2004/04/09 07:47:28  razeto
 * Updated: changed the log level to info, added a string field to the bx_message ctor
 *
 * Revision 1.4  2004/04/06 12:39:28  razeto
 * Added logfile handling
 *
 * Revision 1.3  2004/04/05 13:10:23  razeto
 * Added comments, added an error level, changed some methods
 *
 * Revision 1.2  2004/04/02 14:04:42  razeto
 * Added some comments. Fixed a typo
 *
 */
