/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: vdt.cc,v 1.5 2004/04/09 07:48:19 razeto Exp $
 *
 * Implementation of vdt
 *
 */
#include "vdt.hh"
#include "messenger.hh"

#include <sstream>
#include <stdexcept>
#include <ctype.h>
#include <stdlib.h>

vdt::vdt (const std::string& value, vdt_type type) {
  vdt_type tmp_type = m_search_type (value);

  if (type != unassigned_vdt && tmp_type != type) {
    bx_message msg(bx_message::critic);
    msg << "requested " << char(type) << "type, while \"" << value << "\" has type " << char(tmp_type) << dispatch;
  }

  type_internal = tmp_type;
  m_assign_value (value);
}

const vdt& vdt::operator= (std::string value) {
  vdt_type tmp_type = m_search_type (value);

  m_check_type (tmp_type);

  type_internal = tmp_type;


  m_assign_value (value);

  return *this;
}
  
void vdt::m_check_type (vdt_type new_type) const {
  if (type_internal == unassigned_vdt) return;

  if (new_type == float_vdt && type_internal == int_vdt ||
      new_type == int_vdt && type_internal == float_vdt) return;	// Allow int/float casting

  if (type_internal != new_type) {
    bx_message msg(bx_message::critic);
    msg << "vdt: requested " << char(new_type) << " while curren type is " << char(type_internal) << " (" << *this << ") cast not allowed" << dispatch;
  }
}

vdt::vdt_type vdt::m_search_type (const std::string& str) {
  bool is_float = false;
  bool is_hex = false;
  std::string::size_type i = 0;

  for (; i < str.size (); i++) {
    register char c = str[i];
    if (isdigit (c)) continue;                  // continue on digit
    else if (c == '-' && i == 0) continue;	// negative numbers
    else if (is_hex && isxdigit (c)) continue;  // hex digit are good is is an hex number
    else if (c == 'x') {                        // Allow 0x hex syntax
      if (i != 1 || is_hex) break;
      else is_hex = true;
    } else if (c == '.') {                      // On float only a "." is allowed
      if (is_float) break;
      else is_float = true;
    } else break;
  }

  if (i != str.size ()) return string_vdt;
  else if (is_float) return float_vdt;
  else return int_vdt;
}

void vdt::m_assign_value (const std::string& str) {
  bx_message msg(bx_message::critic, "vdt: ");

  if (type_internal == unassigned_vdt) msg << "internal vdt error: cannot call m_assign_value with a undefined type value" << dispatch;

 if (type_internal == string_vdt) value_s = str;
 else if (type_internal == float_vdt) {
   const char *cstr = str.c_str ();
   char *error_ptr;
   value_f = ::strtod (cstr, &error_ptr);
   if (value_f == 0 && error_ptr == str) msg << "float conversion failed for " << str << dispatch;
 } else {
   const char *cstr = str.c_str ();
   char *error_ptr;
   value_i = ::strtol (cstr, &error_ptr, 0);  // 0 means autodetect the base, which is fine for 0x starting hex
   if (value_i == 0 && error_ptr == str) msg << "integer conversion failed for " << str << dispatch;
 }
} 

std::ostream& operator<< (std::ostream& out, const vdt& v) {
  switch (v.get_type ()) {
    case vdt::int_vdt:
      out << v.get_int ();
      break;
    case vdt::float_vdt:
      out << v.get_float ();
      break;
    case vdt::string_vdt:
      out << v.get_string ();
      break;
    default:
      out << "unknown type"; 
  }
  return out;
}
  
/*
 * $Log: vdt.cc,v $
 * Revision 1.5  2004/04/09 07:48:19  razeto
 * Fixed a bug for negative numbers. Added int->float read casting
 *
 * Revision 1.4  2004/04/05 13:09:07  razeto
 * Added messanger for exception handling. Updated a comment
 *
 * Revision 1.3  2004/04/01 12:05:01  razeto
 * Fixed a bug in the assignement operator.
 * Added a ostreamer function.
 * Added stupid assignement operator since casting is not done.
 *
 * Revision 1.2  2004/03/29 13:23:57  razeto
 * Added the float/int int/float casting
 *
 * Revision 1.1  2004/03/29 13:14:12  razeto
 * Added interface directory (where options and dbi reside).
 * Added vdt object for storing parameters
 *
 */
