/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: vdt.hh,v 1.5 2004/04/09 07:48:19 razeto Exp $
 *
 * Variable Data Type is an object which is constructed from
 * a string and holts the type of the assigne value.
 * vdt("1000") creates a vdt obj holding an integer with value
 * 1000. Later the obj value can be fetced only if the query
 * matches the vdt type, else an exception is generated.
 * A assignement operator is present, which does not allow
 * runtime type change (except float/int).
 *
 */
#ifndef _VDT_H
#define _VDT_H
#include "bx_rec_general.hh"

#include <string>
#include <iostream>

class vdt {
  public:
    enum vdt_type {
      int_vdt = 'I',
      float_vdt = 'F',
      string_vdt = 'S',
      unassigned_vdt = 'U'
    };
    
    vdt (): type_internal(unassigned_vdt) {}
    vdt (const std::string& value, vdt_type type = unassigned_vdt);
    vdt (long int value) { value_i = value; type_internal = int_vdt; }
    vdt (double value)   { value_f = value; type_internal = float_vdt; }

    const vdt& operator= (std::string value); 
    const vdt& operator= (long int value) { m_check_type (int_vdt); type_internal = int_vdt; value_i = value; return *this; }
    const vdt& operator= (double value)   { m_check_type (float_vdt); type_internal = float_vdt; value_f = value; return *this; };
      // Since cast is not called in assignement operator these methods are necessary
    const vdt& operator= (int value)      { return (*this) = (long int)(value); }
    const vdt& operator= (short value)	  { return (*this) = (long int)(value); }
    const vdt& operator= (char value)     { return (*this) = (long int)(value); }
    const vdt& operator= (unsigned long value) { return (*this) = (long int)(value); }
    const vdt& operator= (unsigned short value) { return (*this) = (long int)(value); }
    const vdt& operator= (unsigned char value) { return (*this) = (long int)(value); }
    const vdt& operator= (float value)    { return (*this) = double(value); }

    bool is_valid ()	 const  { return (type_internal == unassigned_vdt) ? 0 : 1; }
    bool operator () ()  const	{ return is_valid (); }
    
    vdt_type get_type () const	{ return type_internal; }
    long int get_int ()  const	{ m_check_type (int_vdt); return value_i; }
    double get_float ()   const	{ m_check_type (float_vdt); return (type_internal == float_vdt) ? value_f : value_i; } // int->float cast always allowed
    const std::string& get_string () const { m_check_type (string_vdt); return value_s; }
    
  private:
    vdt_type type_internal;
    
    long int value_i;
    double value_f;
    std::string value_s;
    
    void m_check_type (vdt_type new_type) const;  // Thows an exception if the internal type is different
						  // from the required type (except if type_internal == unassigned_vdt)
    vdt_type m_search_type (const std::string& str);
    void m_assign_value (const std::string& str);
};

std::ostream& operator<< (std::ostream&, const vdt&);
#endif
/*
 * $Log: vdt.hh,v $
 * Revision 1.5  2004/04/09 07:48:19  razeto
 * Fixed a bug for negative numbers. Added int->float read casting
 *
 * Revision 1.4  2004/04/01 12:05:01  razeto
 * Fixed a bug in the assignement operator.
 * Added a ostreamer function.
 * Added stupid assignement operator since casting is not done.
 *
 * Revision 1.3  2004/03/29 13:26:07  razeto
 * Fixed a typo
 *
 * Revision 1.2  2004/03/29 13:23:27  razeto
 * Fixed a bug, object could be uninitialized
 *
 * Revision 1.1  2004/03/29 13:14:12  razeto
 * Added interface directory (where options and dbi reside).
 * Added vdt object for storing parameters
 *
 */
