/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 * 	   Marco Pallavicini <pallas@ge.infn.it>
 *
 * $Id: bx_base_module.cc,v 1.10 2004/04/24 17:36:45 razeto Exp $
 *
 * Implementation of bx_base_module
 *
 */
#include "bx_base_module.hh"

#include <stdexcept>


// ctor
bx_base_module::bx_base_module (const std::string& myname, bx_base_module::module_role role): s_name(myname), i_role(role) {
  param.clear();
  b_enabled = true;
  i4_priority = 0;

  if (role == bx_base_module::all)
    get_message(bx_message::critic) << "can not istantiate a module with role = all" << dispatch;
}

bx_message& bx_base_module::get_message (bx_message::message_level level) {
  
  message.clear ();

  message.set_level (level); 
  if (check_parameter ("log_level")) message.set_log_level (get_parameter ("log_level"));
  if (check_parameter ("print_level")) message.set_print_level (get_parameter ("print_level"));

  message << "module " << get_name () << ": ";
  return message;
}

void bx_base_module::init_parameter (const std::string& name, const vdt& value) {
  param[name] = value;

  if (name == "enable") b_enabled = value.get_int ();
  else if (name == "priority") i4_priority = value.get_int ();

}
void bx_base_module::set_parameter (const std::string& name, const vdt& value) {
  if (!check_parameter (name) && name != "print_level" && name != "log_level")
    get_message (bx_message::warn) << "assigning value to unknown parameter \"" << name << "\" will initialize it. Check for syntax error" << dispatch;

  init_parameter (name, value);
}

/*
 * $Log: bx_base_module.cc,v $
 * Revision 1.10  2004/04/24 17:36:45  razeto
 * Added a check if the parameter already exist in setting (added an init method too)
 *
 * Revision 1.9  2004/04/06 12:42:17  razeto
 * Added module role to base_module, and the code to handle everywhere. Some other minor fixes
 *
 * Revision 1.8  2004/04/03 09:21:16  razeto
 * changed the message syntax
 *
 * Revision 1.7  2004/04/02 13:39:36  razeto
 * Added messenger. Changed from map to cmap
 *
 * Revision 1.6  2004/03/29 13:15:40  razeto
 * Updated bx_base_module to the vtd usage
 *
 * Revision 1.5  2004/03/26 16:33:59  razeto
 * Added check_parameter to see if a parameter is defined without generating an exception
 *
 * Revision 1.4  2004/03/22 17:26:10  razeto
 * Added module parameter interface
 *
 * Revision 1.3  2004/03/21 19:00:07  razeto
 * Some cosmetic changes
 *
 * Revision 1.2  2004/03/21 18:52:27  razeto
 * Removed "using std::string". Some cosmetic changes
 *
 * Revision 1.1  2004/03/20 17:39:45  pallas
 * First release
 *
 * Revision 1.1  2004/03/20 14:16:47  razeto
 * Added sources
 *
 */
