/* BOREXINO Reconstruction program
 *
 * Author: Davide D'Angelo <davide.dangelo@lngs.infn.it>
 *
 * $Id: bx_muon_decoder.cc,v 1.4 2004/04/28 11:04:49 ddangelo Exp $
 *
 * Implementation of bx_muon_decoder.hh
 * 
*/

#include "bx_muon_decoder.hh"

bx_muon_decoder::bx_muon_decoder() : bx_base_module("bx_muon_decoder", bx_base_module::main_loop) {
}
      
void bx_muon_decoder::begin () {
}

bx_reco_event* bx_muon_decoder::doit (bx_reco_event *ev) {
  const bx_muon_event& er = ev->get_muon();
  // cast to a muon event  
  bx_muon_decoded_event &ew = dynamic_cast<bx_muon_decoded_event&>(ev->get_muon_nonconst());

  float time, charge;
  for (unsigned long i=0 ; i<er.get_raw_nhits(); i++) {    
    const bx_muon_raw_hit& h = er.get_raw_hit(i);
    time = (constants::muon.tdc.time_buffer - h.get_lead_time() /*- cal->time_offset[h.get_muon_channel()] */) * constants::muon.tdc.ns_per_clock; // time in ns from gate start (8192 ticks before trigger==tdc_common_stop) 
    if (h.is_pmt_channel()) {
      charge = (h.get_time_diff() /*- pre->qtc_pedestal[h.get_muon_channel()]*/) * constants::muon.pe_per_clock;
      if (charge < 0) {
	charge = 0;
	// print a warning
      }
    } 
    else // Special channels without pedestals
      charge =  h.get_time_diff() * constants::muon.pe_per_clock;

    ew.decoded_hits.push_back(bx_muon_decoded_hit(time, charge, &h));

    ew.nhits_per_channel[h.get_muon_channel()]++;
  }  
  return ev;  
}

void bx_muon_decoder::end () {
}

/*
 * $Log: bx_muon_decoder.cc,v $
 * Revision 1.4  2004/04/28 11:04:49  ddangelo
 * Modules adapted to call the right const/nonconst event portion getter
 *
 * Revision 1.3  2004/04/27 09:46:50  ddangelo
 * modifications to match new event structure.
 * Other debugging
 *
 * Revision 1.2  2004/04/06 12:42:17  razeto
 * Added module role to base_module, and the code to handle everywhere. Some other minor fixes
 *
 * Revision 1.1  2004/04/02 09:56:03  ddangelo
 * created bx_muon_decoder module.
 * precalibration and calibration/db info are still commented out
 *
 *
 */

