/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: bx_precalib_laben_adc.cc,v 1.5 2004/04/28 11:04:49 ddangelo Exp $
 *
 * Implementation of bx_precalibrator
 *
 */
#include "bx_precalib_laben_adc.hh"
#include "messenger.hh"
#include "laben_time_hit.hh"
#include "constants.hh"
#include "bx_dbi.hh"
#include "db_run.hh"

#include <algorithm>

// ctor
bx_precalib_laben_adc::bx_precalib_laben_adc (): bx_base_module("bx_precalib_laben_adc", bx_base_module::precalib_cycle1) {
}

// module interface
void bx_precalib_laben_adc::begin () {
    // allocate the matrix
  adc_sample_map = new int*[constants::laben.channels]; 
  for (int i = 0; i < constants::laben.channels; i++) adc_sample_map[i] = new int[256](0);

  get_message(bx_message::debug) << "begin" << dispatch;
}

bx_reco_event* bx_precalib_laben_adc::doit (bx_reco_event *ev) {
  const bx_laben_event& er = ev->get_laben();
  for (int i = 0; i < er.get_raw_nhits (); i++) {
    const bx_laben_raw_hit &hit = er.get_raw_hit (i);
    adc_sample_map[hit.get_channel ()][hit.get_time_1()] ++;
    adc_sample_map[hit.get_channel ()][hit.get_time_2()] ++;
  }
  return ev;
}

void bx_precalib_laben_adc::end () {

  db_run& run_info = bx_dbi::get ()->get_run_nonconst (bx_dbi::get ()->get_current_run_number ());
  
  for (int i = 0; i < constants::laben.channels; i++) {
      // calculate local maxima in the first 20 and in the last 20 bins
    int low_bin = std::max_element (adc_sample_map[i], adc_sample_map[i] + 20) - adc_sample_map[i];
    int high_bin = std::max_element (adc_sample_map[i] + 236, adc_sample_map[i] + 256) - adc_sample_map[i];
    
      // add/remove a costant offset
    if (low_bin >= 3) low_bin -= 3;
    if (high_bin <= 252) high_bin += 3;
    
      // if adc_sample_map[i][high_bin] == 0 no hit present
    if (! adc_sample_map[i][high_bin]) high_bin = 255; // low_bin is already at 0 considering max_element engine

      // set the values
    run_info.set_laben_precalib_low_bin (i, low_bin, this);
    run_info.set_laben_precalib_high_bin (i, high_bin, this);
      // deallocate the matrix
    delete [] adc_sample_map[i];
  } 
  delete [] adc_sample_map;

  
  get_message(bx_message::debug) << "end" << dispatch;
}
/*
 * $Log: bx_precalib_laben_adc.cc,v $
 * Revision 1.5  2004/04/28 11:04:49  ddangelo
 * Modules adapted to call the right const/nonconst event portion getter
 *
 * Revision 1.4  2004/04/27 14:25:18  ddangelo
 * Modifications to match new event structure.
 * Useless dynamic_cast in event reading replaced by getter.
 * safer const reference is used instead of non-const pointer.
 *
 * Revision 1.3  2004/04/26 13:53:00  razeto
 * Updated to follow new name/syntax of db_run and db_profile
 *
 * Revision 1.2  2004/04/13 12:09:17  razeto
 * Fixed a bug: the values were not set
 *
 * Revision 1.1  2004/04/12 15:59:49  razeto
 * Added a firsto core of laben precalibration
 *
 */
