/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: bx_precalib_laben_d80.cc,v 1.3 2004/04/28 11:04:49 ddangelo Exp $
 *
 * Implementation of bx_precalibrator
 *
 */
#include "bx_precalib_laben_d80.hh"
#include "messenger.hh"
#include "laben_time_hit.hh"
#include "constants.hh"
#include "bx_dbi.hh"
#include "db_run.hh"
#include "laben_time_hit.hh"

#include <algorithm>

// ctor
bx_precalib_laben_d80::bx_precalib_laben_d80 (): bx_base_module("bx_precalib_laben_d80", bx_base_module::precalib_cycle3) {
}

// module interface
void bx_precalib_laben_d80::begin () {
    // allocate the matrix
  d80_delay_map = new int*[constants::laben.channels]; 
  for (int i = 0; i < constants::laben.channels; i++) d80_delay_map[i] = new int[400](0);

  get_message(bx_message::debug) << "begin" << dispatch;
}

bx_reco_event* bx_precalib_laben_d80::doit (bx_reco_event *ev) {
  const bx_laben_event& er = ev->get_laben();
  for (int i = 0; i < er.get_raw_nhits (); i++) {
    const bx_laben_raw_hit &hit = er.get_raw_hit (i);
    float d80 = laben_time_hit(hit).get_d80 ();
    d80_delay_map[hit.get_channel ()][int ((d80 - 60.) * 10.)] ++;
  }
  return ev;
}

void bx_precalib_laben_d80::end () {

  db_run& run_info = bx_dbi::get ()->get_run_nonconst (bx_dbi::get ()->get_current_run_number ());
  
  for (int i = 0; i < constants::laben.channels; i++) {
      // calculate the histogram mean
    double sum = 0.;
    long int counts = 0;
    for (int j = 0; j < 400; j++) {
       counts += d80_delay_map[i][j];
       sum += (float(j) / 10 + 60) * d80_delay_map[i][j];
    }
    double mean = sum / counts;

    if (counts < 20 || mean < 70. && mean > 90.) {
      if (counts > 20) get_message (bx_message::info) << "d80 not found for channel " << i << " between limits (70:90)" << dispatch;
      mean = 80.;
    }
      // set the value
    run_info.set_laben_precalib_delta80 (i, mean, this);

      // deallocate the matrix
    delete [] d80_delay_map[i];
  } 
  delete [] d80_delay_map;

  
  get_message(bx_message::debug) << "end" << dispatch;
}
/*
 * $Log: bx_precalib_laben_d80.cc,v $
 * Revision 1.3  2004/04/28 11:04:49  ddangelo
 * Modules adapted to call the right const/nonconst event portion getter
 *
 * Revision 1.2  2004/04/27 14:25:18  ddangelo
 * Modifications to match new event structure.
 * Useless dynamic_cast in event reading replaced by getter.
 * safer const reference is used instead of non-const pointer.
 *
 * Revision 1.1  2004/04/26 13:50:51  razeto
 * Added
 *
 */
