/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: bx_precalib_laben_phases.cc,v 1.3 2004/04/28 11:04:49 ddangelo Exp $
 *
 * Implementation of bx_precalibrator
 *
 */
#include "bx_precalib_laben_phases.hh"
#include "messenger.hh"
#include "laben_time_hit.hh"

// ctor
bx_precalib_laben_phases::bx_precalib_laben_phases (): bx_base_module("bx_precalib_laben_phases", bx_base_module::precalib_cycle2) {
}

// module interface
void bx_precalib_laben_phases::begin () {
  for (int i = 0; i < 2400; i++) lg_map[i] = 0;

  get_message(bx_message::debug) << "begin" << dispatch;
}

bx_reco_event* bx_precalib_laben_phases::doit (bx_reco_event *ev) {
  const bx_laben_event& er = ev->get_laben();
  for (int i = 0; i < er.get_raw_nhits (); i++) {
    const bx_laben_raw_hit &hit = er.get_raw_hit (i);
    laben_time_hit t(hit);
    if (t.hw_slope () != t.guess_slope ()) lg_map[hit.get_channel ()] ++;
  }
  return ev;
}

void bx_precalib_laben_phases::end () {
  for (int i = 0; i < 2400; i++) 
    if (lg_map[i] > 100) get_message(bx_message::warn) << "ch " << i << " count " << lg_map[i] << dispatch;

  get_message(bx_message::debug) << "end" << dispatch;
}
/*
 * $Log: bx_precalib_laben_phases.cc,v $
 * Revision 1.3  2004/04/28 11:04:49  ddangelo
 * Modules adapted to call the right const/nonconst event portion getter
 *
 * Revision 1.2  2004/04/27 14:25:18  ddangelo
 * Modifications to match new event structure.
 * Useless dynamic_cast in event reading replaced by getter.
 * safer const reference is used instead of non-const pointer.
 *
 * Revision 1.1  2004/04/12 15:59:49  razeto
 * Added a firsto core of laben precalibration
 *
 */
