/* BOREXINO Reconstruction program
 *
 * Author: Davide D'Angelo <davide.dangelo@lngs.infn.it>
 *
 * $Id: bx_precalib_muon_findpulse.cc,v 1.5 2004/04/28 11:04:49 ddangelo Exp $
 *
 * Implementation of bx_precalib_muon_findpulse.hh
 *
 */

#include "bx_precalib_muon_findpulse.hh"
#include "messenger.hh"
#include "constants.hh"
#include "bx_dbi.hh"
#include "db_run.hh"

const unsigned short bx_precalib_muon_findpulse::pedestal_detect_events = 100; /* Entries for precalib pulse time determination histo */ 

bx_precalib_muon_findpulse::bx_precalib_muon_findpulse(): bx_base_module("bx_precalib_muon_findpulse", bx_base_module::precalib_cycle1) {
  u4_time_buffer = 0;
  u4_hit_counter = 0;
}

void bx_precalib_muon_findpulse::begin () {

  get_message(bx_message::debug) << "begin" << dispatch;
}

bx_reco_event* bx_precalib_muon_findpulse::doit (bx_reco_event *ev) {

  // if we have enough events, do nothing
  if (ev->get_event_number() >= pedestal_detect_events)
    return ev;

  const bx_muon_event& er = ev->get_muon();

  // start looping over hits
  for (unsigned long i = 0; i <er.get_raw_nhits(); i++) { 
    const bx_muon_raw_hit& h = er.get_raw_hit(i);

    // skip special channels
    if (!h.is_pmt_channel())
      continue;

    u4_time_buffer += h.get_lead_time();
    u4_hit_counter++;
      
  } // end of loop over hits

  return ev; 
}

void bx_precalib_muon_findpulse::end () {

  db_run& run_info = bx_dbi::get ()->get_run_nonconst (bx_dbi::get ()->get_current_run_number ());
  
  unsigned long pulse_time = (u4_hit_counter > 0) ? u4_time_buffer / u4_hit_counter : 0;

  run_info.set_muon_precalib_pulse_time(pulse_time, this);
 
  get_message(bx_message::debug) << "end; precalib pulse time " << pulse_time << dispatch;
}

/*
 * $Log: bx_precalib_muon_findpulse.cc,v $
 * Revision 1.5  2004/04/28 11:04:49  ddangelo
 * Modules adapted to call the right const/nonconst event portion getter
 *
 * Revision 1.4  2004/04/27 15:05:53  ddangelo
 * fixed the bug that was causing the crash with no muon data.
 *
 * Revision 1.3  2004/04/27 09:46:50  ddangelo
 * modifications to match new event structure.
 * Other debugging
 *
 * Revision 1.1  2004/04/20 11:36:25  ddangelo
 * added 2 modules for outer muon precalibration
 *
 *
 */
