/* BOREXINO Reconstruction program
 *
 * Author: Marco Pallavicini <pallas@ge.infn.it>
 *
 * $Id: bx_trigger_decoder.cc,v 1.3 2004/04/28 11:04:49 ddangelo Exp $
 *
 * Implementation for bx_trigger_decoder.hh
 * 
*/

#include <cmath>
#include <ctime> 
#include <iostream>
#include "bx_trigger_decoder.hh"

bx_trigger_decoder::bx_trigger_decoder() : bx_base_module("bx_trigger_decoder", bx_base_module::main_loop) {
}
      
void bx_trigger_decoder::begin () {
}

bx_reco_event* bx_trigger_decoder::doit (bx_reco_event *ev) {
  const bx_trigger_event& er = ev->get_trigger();
  // cast to a trigger event
  bx_trigger_decoded_event &ew = dynamic_cast<bx_trigger_decoded_event&>(ev->get_trigger_nonconst());

  // call method to decode GPS data
  gpsclock_decode (er.get_gps1 (), er.get_gps2 (), er.get_gps3 (),
	          ew.i4_day, ew.i4_mon, ew.i4_year, 
	          ew.i4_hour, ew.i4_min, ew.i4_sec, 
		  ew.i4_millisec, ew.i4_microsec,
		  ew.i4_time1, ew.i4_time2);

  if (! (ev->get_event_number () % 100)) { 
    bx_message &msg = get_message(bx_message::debug);
    msg << "Data: " << er.get_day () << "-" << er.get_mon () << "-" << er.get_year ();
    msg << "  Ora: " << er.get_hour () << ":" << er.get_min () << ":" << er.get_sec () << dispatch;
  }

  return ev;  
}

void bx_trigger_decoder::end () {
}

// member function to decode GPS clock raw data info
// accepts as input 5 raw data words (coming from
// the clock )
// returns date and time in 2 formats
// standard one
// number of seconds and nano-seconds since 01-01-2001 at 0:0:0.0000000000
// (the latter is useful to compute time differences among events )
// D. Manuzio 12-11-2001
//            23-03-2004 slight changes to be class member
void bx_trigger_decoder::gpsclock_decode (
    unsigned long g1, unsigned long g2, unsigned long g3, // GPS clock raw data (input)
    int& mday, int& mon, int& year,  // day, month, year (out)
    int& hour, int& min, int& sec,   // hour, minute, second (out)
    int& msec, int& microsec,        // milli-sec and micro-sec (out)
    unsigned long& sec2k, unsigned long& nsec2k  // secs and nano-sec since 01-01-2000 at 0:0:0
    ) {

// YEAR (*year)    current year	
// MDAY (*mday)      1 -> 31	
// MON (*mon)        1 -> 12
// HOUR (*hour)      0 -> 23
// MIN (*min)	     0 -> 59	
// SEC (*sec)	     0 -> 59	
	
  unsigned short wrd;
  int first,second,third,fourth,five;
  int leg,secday,days,bis,i;
  unsigned int nsec;
  int mon_length[12] = {31,28,31,30,31,30,31,31,30,31,30,31};
    
  // compute micro-seconds and nano-seconds
  wrd = (g1>>16)&0xFFFF;
  first = (wrd & 15);
  wrd >>= 4;
  second = (wrd & 15);
  wrd >>= 4;
  third= (wrd & 15);
  wrd >>= 4;
  fourth = (wrd & 15);
  microsec = second*1+third*10+fourth*100;
  nsec = first*100;

  // compute seconds and milli-secs
  wrd = (g1&0xFFFF);
  first = (wrd & 15);
  wrd >>= 4;
  second = (wrd & 15);
  wrd >>= 4;
  third= (wrd & 15);
  
  wrd = ((g2>>16)&0xFFFF);
  fourth = (wrd & 15);
  wrd >>= 4;
  five = (wrd & 7);
  msec = first+second*10+third*100;
  sec =fourth*1+five*10;
  if (sec<0 ||sec>59) 
    std::cerr << "Gps Clock: Error in decoding seconds! " << sec << std::endl;

  // compute minutes 
  wrd = ((g2>>16)&0xFFFF);
  wrd >>= 8;
  first = (wrd & 15);
  wrd >>= 4;
  second = (wrd & 7);
  min = first*1 + second*10;
  if (min<0 ||min>59) 
    std::cerr << "Gps Clock: Error in decoding minutes! " << min << std::endl;

  
  // hour and solar/legal time 
  wrd = (g2&0xFFFF);
  first = (wrd & 15);
  wrd >>= 4;
  second = (wrd & 3);
  wrd >>= 2;
  third = (wrd & 1);
  hour = first*1+second*10;
  if (hour<0 ||hour>23) 
    std::cerr << "Gps Clock: Error in decoding hour! " << hour << std::endl;
  leg = third;

  // month day 
  wrd = (g2&0xFFFF);
  wrd >>= 8;
  first = (wrd & 15);
  wrd >>= 4;
  second = (wrd & 3);
  mday = first*1+second*10;

  // month 
  wrd = g3&0xFFFF;
  first = (wrd & 15);
  wrd >>= 4;
  second = (wrd & 0x1);
  mon = first + second*10;

  // year 
  wrd = g3&0xFFFF;
  wrd >>= 8;
  first = (wrd & 15);
  wrd >>= 4;
  second = (wrd & 15);
  year = first*1+second*10;
  
/*  // compute number of seconds since 01-01-2000  */
  secday = 86400;
  days = year;
  days *= 365;
  if ( year % 4 ) {
    if (mon > 2)
      bis = (year / 4 ) + 1;   // year 2000 bisestile
    else
      bis = (year / 4 );
  }
  else {
    bis = (year / 4 ) + 1;   // year 2000 bisestile
  }

  days += bis;
  for( i=0; i< ((mon)-1); i++ )
    days += mon_length[i];

  days += mday;
  sec2k = secday*(days-1);
  sec2k += (unsigned long)(3600*hour + 60*min + sec);
  nsec2k = (unsigned long)(msec*1000000 + microsec*1000 + nsec);
}

/*
 * $Log: bx_trigger_decoder.cc,v $
 * Revision 1.3  2004/04/28 11:04:49  ddangelo
 * Modules adapted to call the right const/nonconst event portion getter
 *
 * Revision 1.2  2004/04/27 14:27:35  ddangelo
 * modifications to match new event structure.
 * (A getter is now required in event reading)
 * Cnd before dynamic cast in event writing).
 *
 * Revision 1.1  2004/04/27 10:02:48  ddangelo
 * module bx_trigger_decode_module deleted and re-added as bx_trigger_decoder
 *
 *
 */

