/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 * 	   Marco Pallavicini <pallas@ge.infn.it>
 *
 * $Id: bx_writer.cc,v 1.9 2004/04/18 09:15:57 razeto Exp $
 *
 * Implementation of bx_event_writer
 * Prototype with root file output
 *
 */
#include "bx_writer.hh"
#include "bx_root_event.hh"

#include <iostream>
#include <string>

// helper class used to fill root event
void bx_root_event_filler::fill(bx_root_event *ev) {

}

// ctor
bx_event_writer::bx_event_writer (): bx_base_module("bx_event_writer", bx_base_module::writer) {
}

// module interface
void bx_event_writer::begin () {
  std::string name = "echidna.root";
  if (check_parameter ("root_file_name")) name = get_parameter ("root_file_name").get_string ();

  p_root_file = new TFile (name.c_str (), "CREATE");
  if (p_root_file->IsZombie ()) 
    get_message (bx_message::critic) << "root file " << name << " already exists" << dispatch;
  
  p_root_event = new bx_root_event;
  p_root_tree = new TTree ("etree", "Events Tree");
  p_root_tree->Branch ("events", "bx_root_event", &p_root_event);
  
  get_message (bx_message::info) << "root file " << name << " opened" << dispatch;
}

// fill root event using an external visitor
// to avoid bx_root_event to depend on bx_reco_event
bx_reco_event* bx_event_writer::doit (bx_reco_event *ev) {

  // clear root event
  p_root_event->clear ();
  
  // fill root event with new data from reco event
  bx_root_event_filler filler (ev);
  filler.fill (p_root_event);

  // write root event into the TTree
  p_root_tree->Fill ();
  
  return ev;
}

void bx_event_writer::end () {
  if (p_root_file) {
    p_root_file->Write ();
    p_root_file->Close ();
    p_root_file = 0;
    get_message (bx_message::info) << "Root file closed." << dispatch;
  }
}

/*
 * $Log: bx_writer.cc,v $
 * Revision 1.9  2004/04/18 09:15:57  razeto
 * Fixed indentation
 *
 * Revision 1.8  2004/04/16 15:02:22  pallas
 * Start developing of root event structure
 * Not defined yet, but preprared infrastructure
 *
 * Revision 1.7  2004/04/06 12:42:17  razeto
 * Added module role to base_module, and the code to handle everywhere. Some other minor fixes
 *
 * Revision 1.6  2004/04/03 09:20:35  razeto
 * Added messenger
 *
 * Revision 1.5  2004/03/21 18:59:41  razeto
 * Some cosmetic changes
 *
 * Revision 1.4  2004/03/21 18:53:01  razeto
 * Some cosmetic changes
 *
 * Revision 1.3  2004/03/20 18:59:26  pallas
 * Change counters
 *
 * Revision 1.2  2004/03/20 18:55:02  pallas
 * Debugging framework
 *
 * Revision 1.1  2004/03/20 17:39:45  pallas
 * First release
 *
 * Revision 1.1  2004/03/20 14:16:47  razeto
 * Added sources
 *
 */
