/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 * 	   Marco Pallavicini <pallas@ge.infn.it>
 *
 * $Id: bx_writer.hh,v 1.5 2004/04/18 09:15:57 razeto Exp $
 *
 * Special module to write ROOT file and possibly other output formats
 * 
 */
#ifndef _BX_WRITER_H
#define _BX_WRITER_H

#include "TFile.h"
#include "TTree.h"

#include "bx_base_module.hh"
#include "bx_rec_general.hh"
#include "bx_root_event.hh"

// base class for root event filler 
class bx_base_root_event_filler {
  public:
    bx_base_root_event_filler () {}
    virtual ~bx_base_root_event_filler () {}
    
    virtual void fill (bx_root_event*) = 0;
};

// one implemetation of a root event filler
class bx_root_event_filler : public bx_base_root_event_filler {
  public:
    bx_root_event_filler (const bx_reco_event* p) : p_reco_event(p) {}
    virtual ~bx_root_event_filler () {}
    
    // fill root event
    virtual void fill (bx_root_event*);

  private:
    inline const bx_reco_event& reco () const { return *p_reco_event; }
    const bx_reco_event *p_reco_event;
};


class bx_event_writer: public bx_base_module {
  public:
    bx_event_writer ();
    virtual ~bx_event_writer () {}

    virtual void begin ();
    virtual bx_reco_event* doit (bx_reco_event *ev);
    virtual void end ();
	
  private:
    TFile *p_root_file;
    TTree *p_root_tree;
    bx_root_event *p_root_event;
};

#endif
/*
 * $Log: bx_writer.hh,v $
 * Revision 1.5  2004/04/18 09:15:57  razeto
 * Fixed indentation
 *
 * Revision 1.4  2004/04/16 15:02:22  pallas
 * Start developing of root event structure
 * Not defined yet, but preprared infrastructure
 *
 * Revision 1.3  2004/03/21 18:53:01  razeto
 * Some cosmetic changes
 *
 * Revision 1.2  2004/03/20 18:55:02  pallas
 * Debugging framework
 *
 * Revision 1.1  2004/03/20 17:39:45  pallas
 * First release
 *
 * Revision 1.1  2004/03/20 14:16:47  razeto
 * Added sources
 *
 */
